package com.chelvc.framework.export;

import java.io.IOException;
import java.util.Collection;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.common.function.Adapter;
import com.chelvc.framework.common.util.ExcelUtils;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * 数据导出操作接口
 *
 * @author Woody
 * @date 2024/5/9
 */
public interface ExportHandler {
    /**
     * 初始化处理器
     */
    default void initialize() {
    }

    /**
     * 获取导出任务状态
     *
     * @param id 导出任务ID
     * @return 导出任务状态
     */
    Exporting.Status status(long id);

    /**
     * 保存工作薄到文件
     *
     * @param id       导出任务ID
     * @param workbook 工作薄
     * @throws IOException I/O异常
     */
    void save(long id, Workbook workbook) throws IOException;

    /**
     * 下载数据导出文件
     *
     * @param id       导出任务ID
     * @param response Http响应对象
     * @throws IOException I/O操作异常
     */
    void download(long id, HttpServletResponse response) throws IOException;

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param adapter  数据处理回调函数
     * @return 导出任务ID
     */
    default long export(String filename, Adapter<Long, Integer> adapter) {
        return this.export(filename, adapter, (id, volume) -> {
        });
    }

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param adapter  数据处理回调函数
     * @param listener 导出完成监听器
     * @return 导出任务ID
     */
    long export(String filename, Adapter<Long, Integer> adapter, BiConsumer<Long, Integer> listener);

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 导出任务ID
     */
    default <T> long export(String filename, BiFunction<Integer, Integer, Collection<T>> provider,
                            ExcelUtils.Writer<T> writer, String... titles) {
        return this.export(
                filename,
                (id, volume) -> {
                },
                provider, writer, titles
        );
    }

    /**
     * 异步导出数据
     *
     * @param filename 文件名称
     * @param listener 导出完成监听器
     * @param provider 数据生产者
     * @param writer   Excel对象实例写入接口
     * @param titles   表头数组
     * @param <T>      数据类型
     * @return 导出任务ID
     */
    <T> long export(String filename, BiConsumer<Long, Integer> listener,
                    BiFunction<Integer, Integer, Collection<T>> provider, ExcelUtils.Writer<T> writer,
                    String... titles);

    /**
     * 销毁处理器
     */
    default void destroy() {
    }
}
